/*****************************************************************************
* voceisync.h
*
*****************************************************************************/

#if !defined(__VOICESYNC_H__)
# define __VOICESYNC_H__

#include "zaptel_legacy.h" /* Map of Zaptel -> DAHDI definitions */

#define VS_MAJOR_VERSION	"1"
#define VS_MINOR_VERSION	"2.5"	// release minor version
#define VS_PROD_MINOR_VERSION	"2.5.1" // production minor version

#define VS_MAGIC		0x12345678

#define VS_RX_LEN		(8)
#define VS_RXINT_IN_SEC	(1000)		// number of rx int in a sec

typedef struct voicesync_dahdi_ 
{
	struct zt_span		span;
	struct zt_chan*		chan;
	struct zt_chan*		chan_ptrs[1];
	struct zt_chan		_chan;

} voicesync_dahdi_t;


#if defined(CONFIG_VOICESYNC_PCIE)
int voicesync_pcie_register(void);
int voicesync_pcie_unregister(void);
#endif
#if defined(CONFIG_VOICESYNC_PCI)
int voicesync_pci_register(void);
int voicesync_pci_unregister(void);
int voicesync_pci152_register(void);
int voicesync_pci152_unregister(void);
#endif
#if defined(CONFIG_VOICESYNC_USB)
int voicesync_usb_register(void);
int voicesync_usb_unregister(void);
#endif

int voicesync_register(void);
int voicesync_unregister(void);
int voicesync_initialize(voicesync_dahdi_t *dahdi);

#endif /* __VOICESYNC_H__ */
